<?php

namespace Annie\ActivityTask\Providers;

use Illuminate\Support\ServiceProvider;

class ActivityTaskServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'activity_task');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'activity_task');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerActivityTask();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerActivityTask()
    {
        $this->app->bind('activity_task', function($app) {
            return new ActivityTask($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('ActivityTask', 'Lavalite\ActivityTask\Facades\ActivityTask');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('annie.activity_task', function ($app) {
            return $this->app->make('Annie\ActivityTask\ActivityTask');
        });

        // Bind Activity to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityRepository::class
        );        // Bind ActivityActivityArea to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityActivityAreaRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityActivityAreaRepository::class
        );        // Bind ActivityArea to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityAreaRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityAreaRepository::class
        );        // Bind ActivityCollege to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityCollegeRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityCollegeRepository::class
        );        // Bind ActivityCommunity to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityCommunityRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityCommunityRepository::class
        );        // Bind ActivityLog to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityLogRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityLogRepository::class
        );        // Bind ActivityManager to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityManagerRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityManagerRepository::class
        );        // Bind ActivityMetum to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityMetumRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityMetumRepository::class
        );        // Bind ActivityProgramme to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityProgrammeRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityProgrammeRepository::class
        );        // Bind ActivityProject to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityProjectRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityProjectRepository::class
        );        // Bind ActivitySchool to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivitySchoolRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivitySchoolRepository::class
        );        // Bind ActivitySocialPartner to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivitySocialPartnerRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivitySocialPartnerRepository::class
        );        // Bind ActivityWorkArea to repository
        $this->app->bind(
            'Annie\ActivityTask\Interfaces\ActivityWorkAreaRepositoryInterface',
            \Annie\ActivityTask\Repositories\Eloquent\ActivityWorkAreaRepository::class
        );

        $this->app->register(\Annie\ActivityTask\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Annie\ActivityTask\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and activity_task's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/activity_task.php', 'activity_task'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeActivityTask::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['annie.activity_task'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('annie/activity_task.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/activity_task')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/activity_task')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
